import 'package:example/core/base/error_model.dart';
import 'package:example/core/model/todos.dart';
import 'package:flutter/material.dart';
import './pet_view_model.dart';

class PetView extends PetViewModel {
  @override
  Widget build(BuildContext context) {
    // Replace this with your build function
    return Scaffold(
      appBar: AppBar(
        title: Text("Welcome Fireball Example Application"),
      ),
      body: FutureBuilder<List<Todo>>(
        future: fetchPlaceHolderPosts(),
        initialData: [],
        builder: (BuildContext context, AsyncSnapshot<List<Todo>> snapshot) {
          switch (snapshot.connectionState) {
            case ConnectionState.done:
              if (snapshot.hasData) {
                return ListView.builder(
                    itemCount: snapshot.data.length,
                    itemBuilder: (context, index) =>
                        todoCardWidget(snapshot.data[index]));
              } else
                return buildCenterErrorWidget(snapshot.error);
              break;
            case ConnectionState.waiting:
            case ConnectionState.active:
              return buildCenterLoadingWidget();
            default:
              return buildCenterErrorWidget(snapshot.error);
          }
        },
      ),
    );
  }

  Widget todoCardWidget(Todo item) => Card(
        child: ListTile(
          title: Text(item.title),
          leading: Text(item.id.toString()),
          subtitle: Text("User id: ${item.userId}"),
          trailing: doneIconWidget(item.completed),
        ),
      );

  Icon doneIconWidget(bool isDone) {
    return Icon(
      isDone ? Icons.check_circle : Icons.check_circle_outline,
      color: isDone ? Colors.green : Colors.grey,
    );
  }

  Center buildCenterLoadingWidget() {
    return Center(
      child: CircularProgressIndicator(),
    );
  }

  Center buildCenterErrorWidget(BaseError errorResponse) {
    return Center(
      child: Text(errorResponse.message),
    );
  }
}
