import 'package:flutter/material.dart';

import 'config.dart';
import 'learning.dart';
import 'model/header.dart';
import 'model/network_enum.dart';
import 'request.dart';
import 'serializable.dart';

class NetworkManager {
  NetworkLearning learning;
  NetworkConfig config;

  NetworkManager({this.learning, this.config});

  Future get<ResponseType extends Serializable>(String url,
      {@required ResponseType responseType,
      Duration duration,
      bool showLog,
      List<KeyValue> headers,
      List<KeyValue> querys}) {
    return GenericRequestObject<ResponseType>(MethodType.GET, learning, config,
            duration: duration)
        .path(url)
        .responseType(responseType)
        .addHeaders(headers)
        .query(querys)
        .enableLog(showLog)
        .fetch();
  }

  Future post<ResponseType extends Serializable>(String url,
      {@required dynamic body,
      @required ResponseType responseType,
      Duration duration,
      bool showLog,
      List<KeyValue> headers,
      List<KeyValue> querys}) {
    return GenericRequestObject<ResponseType>(MethodType.POST, learning, config,
            body: body, duration: duration)
        .path(url)
        .responseType(responseType)
        .addHeaders(headers)
        .query(querys)
        .enableLog(showLog)
        .fetch();
  }

  Future put<ResponseType extends Serializable>(String url,
      {@required dynamic body,
      @required ResponseType responseType,
      Duration duration,
      bool showLog,
      List<KeyValue> headers,
      List<KeyValue> querys}) {
    return GenericRequestObject<ResponseType>(MethodType.PUT, learning, config,
            body: body, duration: duration)
        .path(url)
        .responseType(responseType)
        .addHeaders(headers)
        .query(querys)
        .enableLog(showLog)
        .fetch();
  }

  Future delete<ResponseType extends Serializable>(String url,
      {@required dynamic body,
      @required ResponseType responseType,
      Duration duration,
      bool showLog,
      List<KeyValue> headers,
      List<KeyValue> querys}) {
    return GenericRequestObject<ResponseType>(
            MethodType.DELETE, learning, config,
            body: body, duration: duration)
        .path(url)
        .responseType(responseType)
        .addHeaders(headers)
        .query(querys)
        .enableLog(showLog)
        .fetch();
  }
}
