import 'dart:io';

import 'package:fireball/fireball.dart';
import 'package:fireball/network/config.dart';
import 'package:fireball/network/learning.dart';
import 'package:fireball/network/local/local_file.dart';
import 'package:fireball/network/manager.dart';
import 'package:fireball/network/model/response_model.dart';
import 'package:fireball/network/model/error_model.dart';
import 'package:flutter_test/flutter_test.dart';

import 'pet_model.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'pet_mock.dart';

// import 'package:fireball/network/local/file.dart';
import 'package:path_provider_platform_interface/path_provider_platform_interface.dart';

import '../pet-io/pet_model.dart';

main() {
  NetworkManager manager;
  LocalFile localFile;
  setUp(() {
    localFile = LocalFile();

    final config = NetworkConfig(baseUrl: "https://petstore.swagger.io/v2/");

    // config.addHeader(KeyValue("SAMPLE", "ALL-REQUEST"));
    // config.setTimeout(Duration(seconds: 10));

    final learning = PetIOLearning();

    manager = NetworkManager(config: config, learning: learning);
    // Need Caching
    SharedPreferences.setMockInitialValues({}); //set values here
    PathProviderPlatform.instance = MockPathProviderPlatform();
  });

  test('removeCache', () async {
    localFile.removeUserRequestSingleCache("pet/1");
    final response = await manager.get<Pet>("pet/2",
        responseType: Pet(), duration: Duration(seconds: 5));

    localFile.removeUserRequestSingleCache("pet/2");

    await Future.delayed(Duration(seconds: 6));
    final response2 = await manager.get<Pet>("pet/2", responseType: Pet());
    expect(response2 is Pet || response2 is BaseError, true);
  });

  test('getFirstDog', () async {
    final response = await manager.get<Pet>("pet/10",
        responseType: Pet(), duration: Duration(minutes: 5));
    final response2 = await manager.get<Pet>("pet/10", responseType: Pet());
    expect(response2 is Pet || response2 is BaseError, true);
  });
}

class BaseError {
  final String data;

  BaseError(this.data);
}

class PetIOLearning extends NetworkLearning {
  @override
  checkCustomError(ErrorModel error) {
    switch (error.statusCode) {
      case HttpStatus.unauthorized:
        return BaseError("Unauthorized");
      case HttpStatus.notFound:
        return BaseError("Not Found");
      default:
    }
    return error.data;
  }

  @override
  checkSuccess<T>(ResponseModel response) {
    print(response.result);
    return response.data;
  }
}
